@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_COMMANDS_CERTIFICATETOPIVCARDCOMMAND_H__
-#define __KLEOPATRA_COMMANDS_CERTIFICATETOPIVCARDCOMMAND_H__
+#pragma once
 
 #include <commands/cardcommand.h>
 
@@ -26,7 +25,7 @@ class CertificateToPIVCardCommand : public CardCommand
 {
     Q_OBJECT
 public:
-    CertificateToPIVCardCommand(const std::string& cardSlot, const std::string &serialno);
+    CertificateToPIVCardCommand(const std::string &cardSlot, const std::string &serialno);
     ~CertificateToPIVCardCommand() override;
 
 public Q_SLOTS:
@@ -44,5 +43,3 @@ private:
 
 }
 }
-
-#endif /* __KLEOPATRA_COMMANDS_CERTIFICATETOPIVCARDCOMMAND_H__ */
