@@ -15,6 +15,7 @@
 #include "aboutdata.h"
 #include "kleopatraapplication.h"
 #include "mainwindow.h"
+#include "utils/migration.h"
 
 #include "accessibility/accessiblewidgetfactory.h"
 
@@ -57,7 +58,6 @@
 #include <QElapsedTimer>
 #include <QEventLoop>
 #include <QMessageBox>
-#include <QTextDocument> // for Qt::escape
 #include <QThreadPool>
 #include <QTime>
 #include <QTimer>
@@ -174,6 +174,8 @@ int main(int argc, char **argv)
     app.init();
     STARTUP_TIMING << "Application initialized";
 
+    Migration::migrate();
+
     QCommandLineParser parser;
     aboutData.setupCommandLine(&parser);
     kleopatra_options(&parser);
