@@ -16,6 +16,11 @@
 
 #include <utils/pimpl_ptr.h>
 
+namespace Kleo
+{
+class KeyListController;
+}
+
 class MainWindow : public KXmlGuiWindow
 {
     Q_OBJECT
@@ -23,8 +28,12 @@ public:
     explicit MainWindow(QWidget *parent = nullptr, Qt::WindowFlags f = {});
     ~MainWindow() override;
 
+    Kleo::KeyListController *keyListController();
+
 public Q_SLOTS:
     void importCertificatesFromFile(const QStringList &files);
+    void exportWindow();
+    void unexportWindow();
 
 protected:
     QByteArray savedGeometry;
