@@ -51,8 +51,8 @@ public:
     Card();
     virtual ~Card();
 
-    virtual bool operator == (const Card &other) const;
-    bool operator != (const Card &other) const;
+    virtual bool operator==(const Card &other) const;
+    bool operator!=(const Card &other) const;
 
     void setStatus(Status s);
     Status status() const;
@@ -66,6 +66,7 @@ public:
     void setDisplaySerialNumber(const QString &sn);
 
     std::string appName() const;
+    QString displayAppName() const;
 
     void setAppVersion(int version);
     int appVersion() const;
@@ -99,23 +100,19 @@ public:
     bool hasNullPin() const;
     void setHasNullPin(bool value);
 
-    bool canLearnKeys() const;
-    void setCanLearnKeys(bool value);
-
     QString errorMsg() const;
     void setErrorMsg(const QString &msg);
 
-    const std::vector<KeyPairInfo> & keyInfos() const;
-    const KeyPairInfo & keyInfo(const std::string &keyRef) const;
+    const std::vector<KeyPairInfo> &keyInfos() const;
+    const KeyPairInfo &keyInfo(const std::string &keyRef) const;
 
     std::string keyFingerprint(const std::string &keyRef) const;
 
 protected:
     void setAppName(const std::string &name);
+    void setDisplayAppName(const QString &displayAppName);
     void setInitialKeyInfos(const std::vector<KeyPairInfo> &infos);
 
-    virtual void processCardInfo();
-
     void addCardInfo(const std::string &name, const std::string &value);
     std::string cardInfo(const std::string &name) const;
 
@@ -125,7 +122,6 @@ private:
     void updateKeyInfo(const KeyPairInfo &keyPairInfo);
 
 private:
-    bool mCanLearn = false;
     bool mHasNullPin = false;
     Status mStatus = NoCard;
     std::string mSerialNumber;
@@ -142,7 +138,7 @@ private:
     QString mErrMsg;
     std::vector<KeyPairInfo> mKeyInfos;
     std::multimap<std::string, std::string> mCardInfo;
+    QString mDisplayAppName;
 };
 } // namespace Smartcard
 } // namespace Kleopatra
-
