@@ -7,7 +7,7 @@
 */
 #include "netkeywidget.h"
 
-#include "keytreeview.h"
+#include "cardkeysview.h"
 #include "kleopatraapplication.h"
 #include "nullpinwidget.h"
 #include "systrayicon.h"
@@ -20,28 +20,25 @@
 #include "commands/changepincommand.h"
 #include "commands/createcsrforcardkeycommand.h"
 #include "commands/createopenpgpkeyfromcardkeyscommand.h"
-#include "commands/detailscommand.h"
-#include "commands/learncardkeyscommand.h"
 
 #include <Libkleo/Algorithm>
 #include <Libkleo/Compliance>
+#include <Libkleo/Debug>
+#include <Libkleo/KeyCache>
+#include <Libkleo/KeyHelpers>
 #include <Libkleo/KeyListModel>
 
-#include <KConfigGroup>
-#include <KSharedConfig>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KSeparator>
 
 #include <QHBoxLayout>
 #include <QInputDialog>
 #include <QLabel>
 #include <QPushButton>
 #include <QScrollArea>
-#include <QTreeView>
 #include <QVBoxLayout>
 
-#include <KLocalizedString>
-#include <KMessageBox>
-
-#include <gpgme++/context.h>
 #include <gpgme++/engineinfo.h>
 
 using namespace Kleo;
@@ -50,20 +47,12 @@ using namespace Kleo::Commands;
 
 NetKeyWidget::NetKeyWidget(QWidget *parent)
     : QWidget(parent)
-    , mSerialNumberLabel(new QLabel(this))
-    , mVersionLabel(new QLabel(this))
-    , mLearnKeysLabel(new QLabel(this))
-    , mErrorLabel(new QLabel(this))
-    , mNullPinWidget(new NullPinWidget(this))
-    , mLearnKeysBtn(new QPushButton(this))
-    , mChangeNKSPINBtn(new QPushButton(this))
-    , mChangeSigGPINBtn(new QPushButton(this))
-    , mTreeView(new KeyTreeView(this))
-    , mArea(new QScrollArea)
 {
     auto vLay = new QVBoxLayout;
 
     // Set up the scroll are
+    mArea = new QScrollArea{this};
+    mArea->setFocusPolicy(Qt::NoFocus);
     mArea->setFrameShape(QFrame::NoFrame);
     mArea->setWidgetResizable(true);
     auto mAreaWidget = new QWidget;
@@ -74,72 +63,36 @@ NetKeyWidget::NetKeyWidget(QWidget *parent)
     scrollLay->addWidget(mArea);
 
     // Add general widgets
+    mVersionLabel = new QLabel{this};
     mVersionLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
     vLay->addWidget(mVersionLabel, 0, Qt::AlignLeft);
 
-    mSerialNumberLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
-
-    auto hLay1 = new QHBoxLayout;
-    hLay1->addWidget(new QLabel(i18n("Serial number:")));
-    hLay1->addWidget(mSerialNumberLabel);
-    hLay1->addStretch(1);
-    vLay->addLayout(hLay1);
+    {
+        auto hLay1 = new QHBoxLayout;
+        auto label = new QLabel(i18nc("@label", "Serial number:"));
+        mSerialNumberLabel = new QLabel{this};
+        mSerialNumberLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+        label->setBuddy(mSerialNumberLabel);
+        hLay1->addWidget(label);
+        hLay1->addWidget(mSerialNumberLabel);
+        hLay1->addStretch(1);
+        vLay->addLayout(hLay1);
+    }
 
+    mNullPinWidget = new NullPinWidget{this};
     vLay->addWidget(mNullPinWidget);
 
-    auto line1 = new QFrame();
-    line1->setFrameShape(QFrame::HLine);
-    vLay->addWidget(line1);
-    vLay->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Certificates:"))), 0, Qt::AlignLeft);
-
-    mLearnKeysLabel = new QLabel(i18n("There are unknown certificates on this card."));
-    mLearnKeysBtn->setText(i18nc("@action", "Load Certificates"));
-    connect(mLearnKeysBtn, &QPushButton::clicked, this, [this]() {
-        mLearnKeysBtn->setEnabled(false);
-        auto cmd = new LearnCardKeysCommand(GpgME::CMS);
-        cmd->setParentWidget(this);
-        cmd->start();
-
-        auto icon = KleopatraApplication::instance()->sysTrayIcon();
-        if (icon) {
-            icon->setLearningInProgress(true);
-        }
-
-        connect(cmd, &Command::finished, this, [icon]() {
-            ReaderStatus::mutableInstance()->updateStatus();
-            icon->setLearningInProgress(false);
-        });
-    });
-
-    auto hLay2 = new QHBoxLayout;
-    hLay2->addWidget(mLearnKeysLabel);
-    hLay2->addWidget(mLearnKeysBtn);
-    hLay2->addStretch(1);
-    vLay->addLayout(hLay2);
-
+    mErrorLabel = new QLabel{this};
     mErrorLabel->setVisible(false);
     vLay->addWidget(mErrorLabel);
 
-    // The certificate view
-    mTreeView->setHierarchicalModel(AbstractKeyListModel::createHierarchicalKeyListModel(mTreeView));
-    mTreeView->setHierarchicalView(true);
+    vLay->addWidget(new KSeparator(Qt::Horizontal));
 
-    connect(mTreeView->view(), &QAbstractItemView::doubleClicked, this, [this](const QModelIndex &idx) {
-        const auto klm = dynamic_cast<KeyListModelInterface *>(mTreeView->view()->model());
-        if (!klm) {
-            qCDebug(KLEOPATRA_LOG) << "Unhandled Model: " << mTreeView->view()->model()->metaObject()->className();
-            return;
-        }
-        auto cmd = new DetailsCommand(klm->key(idx));
-        cmd->setParentWidget(this);
-        cmd->start();
-    });
-    vLay->addWidget(mTreeView);
+    mCardKeysView = new CardKeysView{this};
+    vLay->addWidget(mCardKeysView);
 
     // The action area
-    auto line2 = new QFrame();
-    line2->setFrameShape(QFrame::HLine);
-    vLay->addWidget(line2);
+    vLay->addWidget(new KSeparator(Qt::Horizontal));
     vLay->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Actions:"))), 0, Qt::AlignLeft);
 
     auto actionLayout = new QHBoxLayout();
@@ -163,8 +116,10 @@ NetKeyWidget::NetKeyWidget(QWidget *parent)
         });
     }
 
-    mChangeNKSPINBtn->setText(i18nc("NKS is an identifier for a type of keys on a NetKey card", "Change NKS PIN"));
-    mChangeSigGPINBtn->setText(i18nc("SigG is an identifier for a type of keys on a NetKey card", "Change SigG PIN"));
+    mChangeNKSPINBtn = new QPushButton{this};
+    mChangeNKSPINBtn->setText(i18nc("@action:button NKS is an identifier for a type of keys on a NetKey card", "Change NKS PIN"));
+    mChangeSigGPINBtn = new QPushButton{this};
+    mChangeSigGPINBtn->setText(i18nc("@action:button SigG is an identifier for a type of keys on a NetKey card", "Change SigG PIN"));
 
     connect(mChangeNKSPINBtn, &QPushButton::clicked, this, [this]() {
         doChangePin(NetKeyCard::nksPinKeyRef());
@@ -179,16 +134,9 @@ NetKeyWidget::NetKeyWidget(QWidget *parent)
 
     vLay->addLayout(actionLayout);
     vLay->addStretch(1);
-
-    const KConfigGroup configGroup(KSharedConfig::openConfig(), QStringLiteral("NetKeyCardView"));
-    mTreeView->restoreLayout(configGroup);
 }
 
-NetKeyWidget::~NetKeyWidget()
-{
-    KConfigGroup configGroup(KSharedConfig::openConfig(), QStringLiteral("NetKeyCardView"));
-    mTreeView->saveLayout(configGroup);
-}
+NetKeyWidget::~NetKeyWidget() = default;
 
 namespace
 {
@@ -235,11 +183,6 @@ void NetKeyWidget::setCard(const NetKeyCard *card)
         mChangeSigGPINBtn->setText(i18nc("SigG is an identifier for a type of keys on a NetKey card", "Change SigG PIN"));
     }
 
-    mLearnKeysBtn->setEnabled(true);
-    mLearnKeysBtn->setVisible(card->canLearnKeys());
-    mTreeView->setVisible(!card->canLearnKeys());
-    mLearnKeysLabel->setVisible(card->canLearnKeys());
-
     const auto errMsg = card->errorMsg();
     if (!errMsg.isEmpty()) {
         mErrorLabel->setText(QStringLiteral("<b>%1:</b> %2").arg(i18n("Error"), errMsg));
@@ -248,9 +191,6 @@ void NetKeyWidget::setCard(const NetKeyCard *card)
         mErrorLabel->setVisible(false);
     }
 
-    const auto keys = card->keys();
-    mTreeView->setKeys(keys);
-
     if (mKeyForCardKeysButton) {
         mKeyForCardKeysButton->setEnabled(!card->hasNKSNullPin() && card->hasSigningKey() && card->hasEncryptionKey()
                                           && DeVSCompliance::algorithmIsCompliant(card->keyInfo(card->signingKeyRef()).algorithm)
@@ -259,6 +199,8 @@ void NetKeyWidget::setCard(const NetKeyCard *card)
     if (mCreateCSRButton) {
         mCreateCSRButton->setEnabled(!getKeysSuitableForCSRCreation(card).empty());
     }
+
+    mCardKeysView->setCard(card, NetKeyCard::AppName);
 }
 
 void NetKeyWidget::doChangePin(const std::string &keyRef)
